#include "BulletParticleContainer.h"

CL_OpenGLSurface *BulletParticleContainer::particle;
bool BulletParticleContainer::textureLoaded = false;

// Constructor
BulletParticleContainer::BulletParticleContainer() {

	active = true;

	if (!textureLoaded) {
		particle = new CL_OpenGLSurface("Graphics/bullet_particle", WorldObject::manager);
		textureLoaded = true;
	}
}

// Destructor
BulletParticleContainer::~BulletParticleContainer() {

}

void BulletParticleContainer::paint() {

	particle->bind();

	// Base class handles this
	paintParticles();
}

void BulletParticleContainer::update(float timeScale) {

	time+=timeScale;

	if (time-timeLastParticleAdded > 1/float(ratePerSecond) && active) {

		timeLastParticleAdded = time;

		// ================
		// Add new Particle
		// ================

		Particle newParticle;
		newParticle.setColor(r, g, b);
		newParticle.setSize(18);

		newParticle.totalLife = .5f;
		newParticle.lifeTimeRemaining = .5f;

		newParticle.setLocation(baseObject->x, baseObject->y);
		newParticle.velocity.addVector(Vector2((baseObject->velocity.theta+180) + (rand() % 50 - 25), 15));

		particles[pointer++] = newParticle;

		if (pointer == USUAL_NUMBER_OF_PARTICLES-1)
			pointer = 0;
	}

	for (int i=0; i < USUAL_NUMBER_OF_PARTICLES; i++) {

		if (particles[i].lifeTimeRemaining <= 0) continue;

		// ===================
		// Update the particle
		// ===================

		float percentLifeLeft = particles[i].lifeTimeRemaining/particles[i].totalLife;

		// Jiggle the value around a little so there aren't straight lines between gradations
		float percentLifeLeftModifier = percentLifeLeft + (float(rand() % 4) / 20.0f - .25f);

		// Update position, alpha, and lifetimeremaining
		particles[i].x += (particles[i].velocity*timeScale).getX();
		particles[i].y += (particles[i].velocity*timeScale).getY();
		particles[i].a = percentLifeLeft - .2;
		if (particles[i].a < 0) particles[i].a = 0;
		particles[i].lifeTimeRemaining -= timeScale;
	}
}
