#include "ExhaustParticleContainer.h"

CL_OpenGLSurface *ExhaustParticleContainer::particle;
bool ExhaustParticleContainer::textureLoaded = false;

// Constructor
ExhaustParticleContainer::ExhaustParticleContainer() {

	if (!textureLoaded) {
		particle = new CL_OpenGLSurface("Graphics/exhaust_particle", WorldObject::manager);
		textureLoaded = true;
	}
}

// Destructor
ExhaustParticleContainer::~ExhaustParticleContainer() {

}

void ExhaustParticleContainer::paint() {

	//cout << "Exhaust painting\n";

        WorldObject::glState->set_active();
	particle->bind();

	// Base class handles this
	paintParticles();
}

void ExhaustParticleContainer::update(float timeScale) {

	time+=timeScale;

	if (time-timeLastParticleAdded > 1/float(ratePerSecond) && active) {

		timeLastParticleAdded = time;

		// ================
		// Add new Particle
		// ================

		Particle newParticle;
		newParticle.setColor(1, 1, .5f);
		newParticle.setSize(18);

		newParticle.totalLife = .75f;
		newParticle.lifeTimeRemaining = .75f;

		// Set start point
		Vector2 offset(baseObject->heading+90, rand() % 6 - 3);
		offset.addVector(Vector2(baseObject->heading+180, 28));

		newParticle.setLocation(baseObject->x+offset.getX(), baseObject->y+offset.getY());
		newParticle.velocity.addVector(Vector2((baseObject->heading+180) + (rand() % 50 - 25), 80));

		particles[pointer++] = newParticle;

		if (pointer == USUAL_NUMBER_OF_PARTICLES-1)
			pointer = 0;
	}

	for (int i=0; i < USUAL_NUMBER_OF_PARTICLES; i++) {

		if (particles[i].lifeTimeRemaining <= 0) continue;

		// ===================
		// Update the particle
		// ===================

		float percentLifeLeft = particles[i].lifeTimeRemaining/particles[i].totalLife;

		// Jiggle the value around a little so there aren't straight lines between gradations
		float percentLifeLeftModifier = percentLifeLeft + (float(rand() % 4) / 20.0f - .25f);

		if (percentLifeLeftModifier > .8f) {
			particles[i].r = 1; particles[i].g = 1; particles[i].b = 1;
		} else if (percentLifeLeftModifier > .6f) {
			particles[i].r = .9f; particles[i].g = .9f; particles[i].b = .6f;
		} else if (percentLifeLeftModifier > .4f) {
			particles[i].r = .8f; particles[i].g = .6f; particles[i].b = .4f;
			particles[i].velocity.magnitude *= 1.01f;
		} else if (percentLifeLeftModifier > .2f) {
			particles[i].r = .8f; particles[i].g = .5f; particles[i].b = .4f;
			particles[i].size += (particles[i].size/16);
			particles[i].velocity.magnitude *= 1.01f;
		} else {
			particles[i].r = .5f; particles[i].g = .5f; particles[i].b = .5f;
			particles[i].size += (particles[i].size/14);
			particles[i].velocity.magnitude *= 1.03f;
		}

		// Update position, alpha, and lifetimeremaining
		particles[i].x += (particles[i].velocity*timeScale).getX();
		particles[i].y += (particles[i].velocity*timeScale).getY();
		particles[i].a = percentLifeLeft; 
		particles[i].lifeTimeRemaining -= timeScale;

	}
}
