#include "ExplosionParticleContainer.h"

TexturePtr ExplosionParticleContainer::particle[4];
bool ExplosionParticleContainer::textureLoaded = false;

// Constructor
ExplosionParticleContainer::ExplosionParticleContainer() {

	texture = rand() % 4;

	if (!textureLoaded) {
		particle[0] = new CL_OpenGLSurface("Graphics/fragment_1", WorldObject::manager);
		particle[1] = new CL_OpenGLSurface("Graphics/fragment_2", WorldObject::manager);
		particle[2] = new CL_OpenGLSurface("Graphics/fragment_3", WorldObject::manager);
		particle[3] = new CL_OpenGLSurface("Graphics/fragment_4", WorldObject::manager);
		textureLoaded = true;
	}
}

// Destructor
ExplosionParticleContainer::~ExplosionParticleContainer() {

}

void ExplosionParticleContainer::paint() {

        WorldObject::glState->set_active();

	particle[texture]->bind();

	int currentType = 0;

	// Special explosion paint method
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glTranslatef(SCREENWIDTH/2, SCREENHEIGHT/2, 0);

	glBlendFunc(GL_SRC_ALPHA, GL_ONE);

	glBegin(GL_QUADS);

	for (int i=0; i < USUAL_NUMBER_OF_PARTICLES/2; i++) {

		if (particles[i].lifeTimeRemaining <= 0) continue;

		glColor4f(particles[i].r, particles[i].g, particles[i].b, particles[i].a);
									
		glTexCoord2f(0,0); glVertex2f(-particles[i].size+particles[i].x, -particles[i].size+particles[i].y);
		glTexCoord2f(1,0); glVertex2f(particles[i].size+particles[i].x, -particles[i].size+particles[i].y);
		glTexCoord2f(1,1); glVertex2f(particles[i].size+particles[i].x, particles[i].size+particles[i].y);
		glTexCoord2f(0,1); glVertex2f(-particles[i].size+particles[i].x, particles[i].size+particles[i].y);
	}
		
	glEnd();	
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glColor3f(1, 1, 1);
}

void ExplosionParticleContainer::burst(int number) {
	for (int i=0; i < number; i++) {
		addParticle();
	}
}

void ExplosionParticleContainer::addParticle() {
		// ================
		// Add new Particle
		// ================

		Particle newParticle;

		int particleType = rand() % 3;

		switch (particleType) {

		case TYPE_FRAGMENT: 

			newParticle.setColor(1, 1, 1);
			newParticle.setSize(6);

			newParticle.totalLife = .65f;
			newParticle.lifeTimeRemaining = .45f;

			newParticle.setLocation(baseObject->x, baseObject->y);
			newParticle.velocity.addVector(baseObject->velocity);
			newParticle.velocity.addVector(Vector2(rand() % 360, rand() % 250 + 80));

			break;

		case TYPE_DUST:

			newParticle.setColor(.5f, .5f, .5f);
			newParticle.setSize(32);

			newParticle.totalLife = .75f;
			newParticle.lifeTimeRemaining = .75f;

			// Set start point
			newParticle.setLocation(baseObject->x, baseObject->y);
			newParticle.velocity.addVector(baseObject->velocity*.5f);
			newParticle.velocity.addVector(Vector2(rand() % 360, rand() % 20 + 10));

			break;

		case TYPE_FIRE:

			newParticle.setColor(1, .5f, .5f);
			newParticle.setSize(22);

			newParticle.totalLife = .75f;
			newParticle.lifeTimeRemaining = .75f;

			// Set start point
			newParticle.setLocation(baseObject->x, baseObject->y);
			newParticle.velocity.addVector(baseObject->velocity*.5f);
			newParticle.velocity.addVector(Vector2(rand() % 360, rand() % 40 + 20));

			break;
		}

		particles[pointer++] = newParticle;

		if (pointer == USUAL_NUMBER_OF_PARTICLES-1)
			pointer = 0;
}

void ExplosionParticleContainer::update(float timeScale) {

	time+=timeScale;

	for (int i=0; i < USUAL_NUMBER_OF_PARTICLES; i++) {

		if (particles[i].lifeTimeRemaining <= 0) continue;

		// ===================
		// Update the particle
		// ===================

		float percentLifeLeft = particles[i].lifeTimeRemaining/particles[i].totalLife;

		// Jiggle the value around a little so there aren't straight lines between gradations
		float percentLifeLeftModifier = percentLifeLeft + (float(rand() % 4) / 20.0f - .25f);

		/*
		if (percentLifeLeftModifier > .8f) {
			particles[i].r = 1; particles[i].g = 1; particles[i].b = 1;
		} else if (percentLifeLeftModifier > .6f) {
			particles[i].r = .9f; particles[i].g = .9f; particles[i].b = .6f;
		} else if (percentLifeLeftModifier > .4f) {
			particles[i].r = .8f; particles[i].g = .6f; particles[i].b = .4f;
			particles[i].velocity.magnitude *= 1.01f;
		} else if (percentLifeLeftModifier > .2f) {
			particles[i].r = .8f; particles[i].g = .5f; particles[i].b = .4f;
			particles[i].size += (particles[i].size/16);
			particles[i].velocity.magnitude *= 1.01f;
		} else {
			particles[i].r = .5f; particles[i].g = .5f; particles[i].b = .5f;
			particles[i].size += (particles[i].size/14);
			particles[i].velocity.magnitude *= 1.03f;
		}*/

		// Update position, alpha, and lifetimeremaining
		particles[i].x += (particles[i].velocity*timeScale).getX();
		particles[i].y += (particles[i].velocity*timeScale).getY();
		particles[i].a = percentLifeLeft; 
		particles[i].lifeTimeRemaining -= timeScale;

	}
}

